
#include <math.h>
#include "general.h"
#include "Materials.h"

RGB_Color mWhite=RGB_Color(1,1,1);
RGB_Color mBlack=RGB_Color(0,0,0);

float wrap(float c)
{
  if(c>=0.0f){
    return      fract(c);
  } else {
	return 1.0f-fract(c);
  }
}


// fresnel formula optimized by Dmitry Omelchenko aka tigra:
float R_fnf(float n1, float n2, double ce1)
{
  double c12,z,A2B2,AB2,C2,D2,ZC12,face1;

  if(fabs(n2-n1)<0.001) return 0.0f;
  face1=fabs(ce1);
  if(acos(face1)<0.01f) return (float)sqr((n1-n2)/(n1+n2));
  z=1-face1*face1;
  c12=z*sqr(n1/n2);
  if(c12<1.0){
    ZC12=z*c12;
    A2B2=z+c12-ZC12-ZC12;
    AB2=sqrt((z-ZC12)*(c12-ZC12));AB2=AB2+AB2;
    C2=A2B2-AB2;D2=A2B2+AB2;
    return (float) (C2*(1+(1-D2)/(1-C2))/(D2+D2));
  }
  else return 1.0f;
}

float Material::IOR(float w)
{
  if(w==0.0 || IOR_offset<0.0)
	return IOR_base;
  else
	return IOR_base + IOR_offset / (w - IOR_sub);
}

float Material::IOR()
{
	return IOR_base;
}

RGB_Color Material::Diff(float x, float y)
{
//  if((Specularity > 0.99) || (Reflection.R < 0.0f)) return mBlack;
  if(Texture){
    unsigned char *bitptr = Texture->tbits + ((int)(wrap(y)*Texture->tinfo.bmiHeader.biHeight)) * ((Texture->tinfo.bmiHeader.biWidth*((Texture->tinfo.bmiHeader.biBitCount)>>3)+3) & 0xFFFFFFFC);
    return (1.0f - Specularity) * RGB_Color(bitptr[3*((int)(wrap(x)*Texture->tinfo.bmiHeader.biWidth))+2]/255.0f, bitptr[3*((int)(wrap(x)*Texture->tinfo.bmiHeader.biWidth))+1]/255.0f, bitptr[3*((int)(wrap(x)*Texture->tinfo.bmiHeader.biWidth))]/255.0f);
  }
  if(Reflection.G < 0.0f) 
	  return (1.0f - Specularity) * Reflection.R * mWhite;
  return (1.0f - Specularity) * Reflection;
}

/*
RGB_Color Material::Spec()
{
  if(Specularity < 0.01) 
	  return mBlack;
  return Absorption;
}
*/
/*
RGB_Color Material::Trns(float n1, float n2, double ce1)
{
  if((Specularity < 0.01) || ((Transmission.R2() < 0.01f) && (Reflection.R2() >= 0.0f)) ) 
	  return mBlack;
  if(Reflection.R < 0.0f) 
	  return (1.0f - R_fnf(n1,n2,ce1))*mWhite;
  return Transmission.R2() * mWhite;
}
*/
/*
void Material::Fix(void)
{
	if (Emission.R<0) {
		Emission.R =0;
		Emission.G =0;
		Emission.B =0;
	}
	if (Emission.G<0) {
		Emission.R = Emission.R;
		Emission.G = Emission.R;
		Emission.B = Emission.R;
	}
}
*/
RGB_Color Material::Thru(float d)
{
//  if(Absorption.G < 0.0f) 
//	  if(Absorption.R==0.0f) 
//		  return mWhite; 
//	  else 
//		  return (float)pow((1.0f-Absorption.R), d/Ref_Length)*mWhite;
  return  
	  RGB_Color(
	  (float)pow((1.0f-Absorption.R), d/Ref_Length),
	  (float)pow((1.0f-Absorption.G), d/Ref_Length),
	  (float)pow((1.0f-Absorption.B), d/Ref_Length));
}
RGB_Color Material::Thru(vector *a,vector *b)
{
  if(Absorption.G < 0.0f) 
	  if(Absorption.R==0.0f) 
		  return mWhite; 
	  else 
		  return ((float)pow((1.0f-Absorption.R),(float)distance(*a, *b)/Ref_Length))*mWhite;
  float d=distance(*a,*b);
  return  
	  RGB_Color(
	  (float)pow((1.0f-Absorption.R), d/Ref_Length),
	  (float)pow((1.0f-Absorption.G), d/Ref_Length),
	  (float)pow((1.0f-Absorption.B), d/Ref_Length));
}
